#!/bin/bash

#############################################################
# Metiix Blockade Server for Linux Installer Script
#############################################################

BASEDIR=`pwd`
ROOT_USER=n

OPTDIR=/opt/blockade
ETCDIR=/etc
INITDIR=/etc/init.d
PARENT_INSTALL_DIR=${OPTDIR}
RUNLEVEL=`who -r | sed -e 's/^.*\(run-level [0-9]\).*$/\1/' | cut -d \  -f 2`
RUNLVLDIR=/etc/rc${RUNLEVEL}.d
BINSLOC="/bin /usr/bin /usr/local/bin"
REQDBINS="sed tar gzip cut who"
RECOMMENDED_DIR=~
USERNAME="`id -un`"

usage() {
		echo "Usage: ./install.sh"
}

prompt_input() {
		local __resultvar=$1
		local __result=$2

		read -p "$3 [$2] " tmp_read
		[[ "_$tmp_read" != "_" ]] && __result="$tmp_read"
		eval $__resultvar="'$__result'"
}

prepdir() {
if [ ! -d "${1}" ] ; then
	echo "${1} does not exist."
		prompt_input YN_PD "y" "Create $1? (y/n)"
	if [ "${YN_PD}" == "y" ] ; then
		mkdir -p "${1}"
		if [ $? -ne 0 ] ; then
			echo "Failed to create ${1}.  Please check your configuration."
			return 1
		fi
	else
		echo "Please check your configuration."
		return 1
	fi
fi
return 0
}

SAVED_ARGS=( "$@" )

if [ "${1}" != "recall" ]; then
	echo ""
	echo "Welcome to the Metiix Blockade Server Installer."
	echo ""
	echo -n "Press enter to continue with installation. "
	read ENTER

	# Basic requirements -
	echo ""
	echo "Validating environment..."
fi

agreed=0
while [ "${agreed}" == "0" ] ; do
	if [ "${USERNAME}" != "root" ]; then
		echo ""
		echo "You are apparently not installing as root."
		echo "current user: ${USERNAME}."
		echo ""

				DEFAULT="y"
				prompt_input YN "$DEFAULT" "Would you like to switch users and install as root? (y/n)"

		case ${YN} in
			[yY] | [yY][eE][sS])
				echo ""
				echo "Switching to root to continue installation"
				sudo ${0} recall ${SAVED_ARGS[@]}
				exit 0
				;;
			[nN] | [nN][oO])
				echo "Exiting."
				return 1
				;;
		esac
	else
		echo "    detected root install"
		ROOT_USER=y
		agreed=1
	fi
done

echo "  reviewing required executables ..."
echo -n "    "
for BIN in $REQDBINS ; do

	BIN_PATH=`which $BIN 2> /dev/null`
	if [[ $? != 0 ]]; then
		echo "$BIN not found and is required. Installation will exit."
		exit 1
	fi
done
echo ""

ARCHIVE="`pwd`/`ls MetiixBlockade*.tgz`"

INTERVIEW=0
while [ ${INTERVIEW} == 0 ] ; do

INTERVIEWSUB=0
while [ ${INTERVIEWSUB} == 0 ] && [ "${ROOT_USER}" != "n" ] ; do
	echo ""
			prompt_input INITDIR_X "$INITDIR" "What directory contains your SYSV init scripts?"
	if [ "x${INITDIR_X}" != "x" ] ; then
		INITDIR=${INITDIR_X}
	fi
	prepdir ${INITDIR}
	if [ $? == 0 ] ; then
		INTERVIEWSUB=1
	fi
done

INTERVIEWSUB=0
while [ ${INTERVIEWSUB} == 0 ] && [ "${ROOT_USER}" != "n" ] ; do
	echo ""
	echo -n
			prompt_input RUNLVLDIR_X "${RUNLVLDIR}" "What directory contains your runlevel init links?"
	if [ "x${RUNLVLDIR_X}" != "x" ] ; then
		RUNLVLDIR=${RUNLVLDIR_X}
	fi
	prepdir ${RUNLVLDIR}
	if [ $? == 0 ] ; then
		INTERVIEWSUB=1
	fi
done

echo ""
	echo ""
	echo "Metiix Blockade Server will install to:"
	echo "  ${PARENT_INSTALL_DIR}"

	if [ "${ROOT_USER}" == "y" ] ; then
		echo "Your selections:"
		echo "  init.d dir is: ${INITDIR}"
		echo "  runlevel directory is: ${RUNLVLDIR}"
	fi

	echo ""
		prompt_input reply "y" "Are these settings correct? (y/n)"

	case ${reply} in
	[yY] | [yY][eE][sS])
		INTERVIEW=1
		;;
	[nN] | [nN][oO])
		echo "Options are incorrect, restarting configuration.";
		;;
	esac
done

if [ -f ${OPTDIR}/.install.vars ]; then
	echo "Metiix Blockade Server appears to already be installed here:"
	echo "  ${OPTDIR}"
	echo "If reinstalling, please uninstall first and then install again."
	echo "./uninstall.sh -i ${OPTDIR}"
	exit 1
fi

echo ""
echo "Installing the Blockade Server ..."
echo "  creating standard directories"
mkdir -p ${OPTDIR} || { echo "Failed to make required directory ${OPTDIR}. Exiting."; exit 1; }
mkdir -p ${INITDIR}
mkdir -p ${RUNLVLDIR}

echo "  recording install configuration"
echo "" > ${OPTDIR}/.install.vars
echo "OPTDIR=${OPTDIR}" >> ${OPTDIR}/.install.vars
echo "ETCDIR=${ETCDIR}" >> ${OPTDIR}/.install.vars
echo "INITDIR=${INITDIR}" >> ${OPTDIR}/.install.vars
echo "RUNLVLDIR=${RUNLVLDIR}" >> ${OPTDIR}/.install.vars
NOW=`date +%Y%m%d`
echo "INSTALLDATE=$NOW" >> ${OPTDIR}/.install.vars

echo "  unpacking ${ARCHIVE}"
HERE=`pwd`
cd ${OPTDIR}
tar -xozf ${ARCHIVE}
cd ${HERE}

cp ./blockade_init.proto ${INITDIR}/blockade
chmod 755 ${INITDIR}/blockade

echo "  linking start script to ${RUNLVLDIR}/S02blockade"
ln -s ${INITDIR}/blockade ${RUNLVLDIR}/S02blockade

echo "  correcting permissions"
chmod 750 $OPTDIR
find $OPTDIR -type d -exec chmod 750 {} \;
chmod 700 $OPTDIR/blockade*
chmod 640 $OPTDIR/.install.vars
echo ""
${INITDIR}/blockade start

echo ""
echo ""
echo ""
echo "Installation complete!"
echo ""
echo "At this point you are able to open the web console at http://localhost:8053."
echo ""
echo "Start/Stop Scripts:"
echo "  ${INITDIR}/blockade"
echo ""
